﻿using System;
using System.Diagnostics;
using System.IO;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using WindowsAPICodePack.Dialogs;

namespace fisp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private PackageHandler? _packageHandler;
        public bool IsEmbedded => ComboBoxChannel.SelectedIndex == 0;

        public MainWindow()
        {
            InitializeComponent();
            UploadingProgress.Minimum = 0;
            UploadingProgress.Maximum = 100;
            TotalProgress.Minimum = 0;
            TotalProgress.Maximum = 100;
            TopPanel.SelectionChanged += OnTabControlSelectionChanged;
            ComboBoxChannel.SelectionChanged += OnChannelSelectionChanged;
        }

        private void OnChooseSourceFolderClick(object sender, RoutedEventArgs e)
        {
            using CommonOpenFileDialog dialog = new()
            {
                IsFolderPicker = true,
            };
            if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
            {
                SourceFolder.Text = dialog.FileName;
                _packageHandler = new PackageHandler(SourceFolder.Text, IsEmbedded);
                PackageName.Text = _packageHandler.FullBuildName;
                PackageFolder.Visibility = Visibility.Collapsed;
                if (_packageHandler.FullBuildName == "Error!")
                {
                    _packageHandler = null;
                    PackageButton.IsEnabled = false;
                    ResetPublishStep();
                    PublishStep2.IsEnabled = false;
                    MessageBox.Show(this, $"Please choose correct flutter web folder!", "Choose folder", MessageBoxButton.OK, MessageBoxImage.Error);
                }
                else
                {
                    PackageButton.IsEnabled = true;
                }
            }
        }

        private void OnPackageClick(object sender, RoutedEventArgs e)
        {
            if (_packageHandler == null)
            {
                return;
            }
            BusyIndicator.IsBusy = true;
            BusyIndicator.BusyContent = "Packaging...";

            Task.Run(async () =>
            {
                try
                {
                    await _packageHandler.Handle();
                    await _packageHandler.GenerateZip();
                    Dispatcher.Invoke(() =>
                    {
                        PackageFolder.Visibility = Visibility.Visible;
                        PackageFolder.Text = PackageHandler.WORKSPACE;
                        BusyIndicator.IsBusy = false;
                        if (!IsEmbedded)
                        {
                            PublishButton.IsEnabled = true;
                        }
                        MessageBox.Show(this, "Package successed!", "Package result", MessageBoxButton.OK, MessageBoxImage.Information);
                    });
                }
                catch (Exception ex)
                {
                    BusyIndicator.IsBusy = false;
                    MessageBox.Show(this, $"Package failed! \n\n {ex.Message}", "Package result", MessageBoxButton.OK, MessageBoxImage.Error);
                }

            });

        }

        private void OnPlulishClick(object sender, RoutedEventArgs e)
        {
            if (_packageHandler == null) return;

            LockActions();
            libfisp.fisp publisher = new libfisp.fisp();
            try
            {
                var folder = _packageHandler.DistFolder;
                var task = publisher.UploadDir(folder, _packageHandler.FullBuildName);
                task.UploadingProgress += (s, e) =>
                {
                    Dispatcher.Invoke(() =>
                    {
                        UploadingProgressText.Text = $"Uploading file {e.File} ... {(int)e.Progress}%";
                        UploadingProgress.Value = e.Progress;
                    });
                };
                task.TotalProgress += (s, e) =>
                {
                    Dispatcher.Invoke(() =>
                    {
                        TotalProgressText.Text = $"Total progress ... {(int)e}%";
                        TotalProgress.Value = e;
                    });
                };
                task.Successed += (s, e) =>
                {
                    Dispatcher.Invoke(() =>
                    {
                        PublishUrl.Text = $"https://app.fis.plus/{PackageName.Text}/index.html";
                        PublishUrl.Visibility = Visibility.Visible;
                        UnlockActions();
                        MessageBox.Show(this, "Publish successed!", "Publish result", MessageBoxButton.OK, MessageBoxImage.Information);
                    });
                };
                task.Failed += (s, m) =>
                {
                    Dispatcher.Invoke(() =>
                    {
                        UnlockActions();
                        MessageBox.Show(this, $"Publish failed:\n\n{m}", "Publish result", MessageBoxButton.OK, MessageBoxImage.Error);
                    });
                };
            }
            catch (Exception ex)
            {
                UnlockActions();
                MessageBox.Show(this, $"Publish failed:\n\n{ex}", "Publish result", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void OnChannelSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            PackageFolder.Visibility = Visibility.Collapsed;
            if (_packageHandler != null)
            {
                _packageHandler.IsEmbedded = IsEmbedded;
            }
            ResetPublishStep();
            PublishStep2.IsEnabled = !IsEmbedded;
        }

        private void ResetPublishStep()
        {
            PublishButton.IsEnabled = false;
            UploadingProgressText.Text = string.Empty;
            TotalProgressText.Text = string.Empty;
            UploadingProgress.Value = 0;
            TotalProgress.Value = 0;
            PublishUrl.Visibility = Visibility.Collapsed;
        }

        private void OnPackageNameChanged(object sender, TextChangedEventArgs e)
        {
            if (PublishUrl != null)
            {
                PublishUrl.Text = $"https://app.fis.plus/{PackageName.Text}/index.html";
                PublishStep2.IsEnabled = true;
            }
        }


        /// <summary>
        /// 锁住所用操作
        /// </summary>
        private void LockActions()
        {
            TopPanel.IsEnabled = false;
        }

        /// <summary>
        /// 解锁所有操作
        /// </summary>
        private void UnlockActions()
        {
            TopPanel.IsEnabled = true;
        }

        private void OnTabControlSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.Source.GetType() != typeof(TabControl))
            {
                e.Handled = true;
                return;
            }

            var tabControl = (TabControl)sender;
            if (tabControl.SelectedIndex == 1)
            {
                BusyIndicator.IsBusy = true;
                OnlineFolderList.Items.Clear();
                Task.Run(() =>
                {
                    libfisp.fisp publisher = new libfisp.fisp();
                    try
                    {
                        var folders = publisher.ListFolders(string.Empty);
                        Dispatcher.Invoke(() =>
                        {
                            foreach (var folder in folders)
                            {
                                OnlineFolderList.Items.Add(folder.Replace("/", ""));
                            }
                            BusyIndicator.IsBusy = false;
                        });
                    }
                    catch (Exception ex)
                    {
                        Dispatcher.Invoke(() =>
                        {
                            BusyIndicator.IsBusy = false;
                            MessageBox.Show(this, $"Load online folders failed:\n\n{ex.Message}", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                        });
                    }
                });
            }
            else
            {
                OnlineFolderList.Items.Clear();
            }
        }

        private void OnDeleteFolderClick(object sender, MouseButtonEventArgs e)
        {
            var obj = (TextBlock)sender;
            var folder = (string)obj.DataContext;
            if (MessageBox.Show(this, "Are you sure you want to delete this folder?", "Warning", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                LockActions();
                BusyIndicator.IsBusy = true;
                libfisp.fisp publisher = new libfisp.fisp();
                try
                {
                    var task = publisher.DeleteFolder(folder + "/");
                    task.Progress += (s, e) =>
                    {
                        Dispatcher.Invoke(() =>
                        {
                            BusyIndicator.BusyContent = $"Deleting folder {folder} ... {(int)e}%";
                        });
                    };
                    task.Successed += (s, e) =>
                    {
                        Dispatcher.Invoke(() =>
                        {
                            OnlineFolderList.Items.Remove(folder);
                            BusyIndicator.IsBusy = false;
                            UnlockActions();
                        });
                    };
                    task.Failed += (s, m) =>
                    {
                        Dispatcher.Invoke(() =>
                        {
                            BusyIndicator.IsBusy = false;
                            UnlockActions();
                            MessageBox.Show(this, $"Delete folder {folder} failed:\n\n{m}", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                        });
                    };
                }
                catch (Exception ex)
                {
                    BusyIndicator.IsBusy = false;
                    UnlockActions();
                    MessageBox.Show(this, $"Delete folder {folder} failed:\n\n{ex.Message}", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        private void OnPackageConsoleMouseDown(object sender, MouseButtonEventArgs e)
        {
            Process.Start("explorer.exe", PackageFolder.Text);
        }
    }
}
