﻿using fisp.Models;
using System;
using System.IO;
using System.IO.Compression;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Windows;

namespace fisp
{
    internal class PackageHandler
    {
        public static readonly string WORKSPACE = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "FisPublish");
        private const string CDN_ROOT = "https://app.fis.plus/";
        private const string GOOGLE_FONTS_CDN = "https://static.fis.plus/flyinsono/GoogleFonts/";
        private static string GOOGLE_FONT_PATH = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resources/GoogleFonts.zip");

        private FlutterVersionModel? _versionModel;

        internal PackageHandler(string folder, bool isEmbedded)
        {
            Folder = folder;
            IsEmbedded = isEmbedded;
            DistFolder = Path.Combine(WORKSPACE, isEmbedded ? "dist_embedded" : "dist_public");
        }

        /// <summary>
        /// 资源目录
        /// </summary>
        public string Folder { get; set; }

        /// <summary>
        /// 是否自建包
        /// </summary>
        public bool IsEmbedded { get; set; }

        /// <summary>
        /// 输出目录
        /// </summary>
        public string DistFolder { get; private set; }

        /// <summary>
        /// 完整编译版本
        /// </summary>
        public string FullBuildName
        {
            get
            {
                if (_versionModel == null)
                {
                    try
                    {
                        LoadVersion();
                    }
                    catch
                    {
                        return "Error!";
                    }
                }
                return $"{_versionModel!.Version}.{_versionModel.BuildNumber}";
            }
        }

        /// <summary>
        /// 打包前处理
        /// </summary>
        /// <returns></returns>
        internal async Task Handle()
        {
            CreateCleanFolder(DistFolder);
            CopyDirectory(Folder, DistFolder);

            if (IsEmbedded)
            {
                await UnzipGoogleFonts();
            }

            //await GenerateVersionFolder();

            await Task.Delay(1000);

            await UpdateMainDartJs();
            await UpdateIndexLinks();
        }

        /// <summary>
        /// 打包 - 创建Zip包
        /// </summary>
        /// <returns></returns>
        internal async Task GenerateZip()
        {
            await Task.Run(() =>
            {
                if (IsEmbedded)
                {
                    var fileName = Path.Combine(WORKSPACE, $"embedded_{FullBuildName}.zip");
                    if (File.Exists(fileName))
                    {
                        File.Delete(fileName);
                    }
                    ZipFile.CreateFromDirectory(DistFolder, fileName);
                }
                else
                {
                    var fileName = Path.Combine(WORKSPACE, $"public_{FullBuildName}.zip");
                    if (File.Exists(fileName))
                    {
                        File.Delete(fileName);
                    }
                    using (ZipArchive archive = ZipFile.Open(fileName, ZipArchiveMode.Create))
                    {
                        AddFileToZip(archive, Path.Combine(DistFolder, "index.html"));
                        AddFileToZip(archive, Path.Combine(DistFolder, "flutter_service_worker.js"));

                        AddFolderToZip(archive, Path.Combine(DistFolder, "assets"));
                    }
                }
            });
        }

        /// <summary>
        /// 更新main.dart.js，替换google字体路径
        /// </summary>
        /// <param name="staticRoot"></param>
        private async Task UpdateMainDartJs()
        {
            // example: https://static.fis.plus/flyinsono/GoogleFonts/notoemoji/v32/bMrnmSyK7YY-MEu6aWjPDs-ar6uWaGWuob-r0jwvS-FGJCMY.ttf

            var staticRoot = IsEmbedded ? "./GoogleFonts/" : GOOGLE_FONTS_CDN;
            var filePath = Path.Combine(DistFolder, "main.dart.js");
            string content = File.ReadAllText(filePath, Encoding.UTF8);

            content = content.Replace("https://fonts.gstatic.com/s/", staticRoot);

            File.Delete(filePath);

            using (var fs = new FileStream(filePath, FileMode.CreateNew, FileAccess.Write, FileShare.None))
            {
                var buffer = Encoding.UTF8.GetBytes(content);
                await fs.WriteAsync(buffer);
            }
        }

        /// <summary>
        /// 更新索引页链接路径
        /// </summary>
        private async Task UpdateIndexLinks()
        {
            var staticRoot = IsEmbedded ? "./" : $"{CDN_ROOT}{FullBuildName}/";
            var filePath = Path.Combine(DistFolder, "index.html");

            string content = File.ReadAllText(filePath, Encoding.UTF8);

            content = content
                .Replace("src=\"js/", $"src=\"{staticRoot}js/")
                .Replace("href=\"style/", $"href=\"{staticRoot}style/")
                .Replace("href=\"flyinsono.ico", $"href=\"{staticRoot}flyinsono.ico")
                .Replace("window.STATIC_ROOT = './';", $"window.STATIC_ROOT = '{staticRoot}';");

            File.Delete(filePath);

            using (var fs = new FileStream(filePath, FileMode.CreateNew, FileAccess.Write, FileShare.None))
            {
                var buffer = Encoding.UTF8.GetBytes(content);
                await fs.WriteAsync(buffer);
            }
        }

        private async Task GenerateVersionFolder()
        {
            // TODO: 支持asset自定义root后，拷贝Version文件夹
            await Task.Run(() =>
            {
                var versionFolder = Path.Combine(DistFolder, FullBuildName);
                CreateCleanFolder(versionFolder);

                if (IsEmbedded)
                {
                    FillEmbeddedVersionFolder(versionFolder);
                }
                else
                {
                    FillPublicVersionFolder(versionFolder);
                }
            });
        }

        private void FillEmbeddedVersionFolder(string folder)
        {

        }

        private void FillPublicVersionFolder(string folder)
        {
            var assetsFoloder = Path.Combine(DistFolder, "assets");
            var assetsDistFolder = Path.Combine(folder, "assets");
            CreateCleanFolder(assetsDistFolder);
            CopyDirectory(assetsFoloder, assetsDistFolder);
        }

        private async Task UnzipGoogleFonts()
        {
            await Task.Run(() =>
            {
                string zipFilePath = GOOGLE_FONT_PATH;
                string extractPath = DistFolder;
                ZipFile.ExtractToDirectory(zipFilePath, extractPath);
            });
        }

        private void LoadVersion()
        {
            var versionJsonPath = Path.Combine(Folder, "version.json");
            var jsonText = File.ReadAllText(versionJsonPath, Encoding.UTF8);
            _versionModel = JsonSerializer.Deserialize<FlutterVersionModel>(jsonText);
        }

        private static void AddFolderToZip(ZipArchive archive, string folder, string? rootFolder = null)
        {
            var root = rootFolder ?? Path.GetDirectoryName(folder);
            var subStrIndex = root.Length + 1;
            DirectoryInfo d = new DirectoryInfo(folder);
            FileInfo[] files = d.GetFiles("*");
            foreach (FileInfo file in files)
            {
                var targetFolder = file.FullName.Substring(subStrIndex);
                archive.CreateEntryFromFile(file.FullName, targetFolder);
            }

            DirectoryInfo[] directories = d.GetDirectories();
            foreach (DirectoryInfo directory in directories)
            {
                AddFolderToZip(archive, directory.FullName, root);
            }
        }

        private static void AddFileToZip(ZipArchive archive, string path)
        {
            var entryName = Path.GetFileName(path);
            archive.CreateEntryFromFile(path, entryName);
        }

        /// <summary>
        /// 创建空文件夹
        /// </summary>
        /// <param name="folder"></param>
        public static void CreateCleanFolder(string folder)
        {
            if (Directory.Exists(folder))
            {
                ClearFolder(folder);
            }
            else
            {
                Directory.CreateDirectory(folder);
            }
        }

        /// <summary>
        /// 清空文件夹
        /// </summary>
        /// <param name="folder"></param>
        public static void ClearFolder(string folder)
        {
            if (!Directory.Exists(folder))
            {
                return;
            }

            // 清空文件夹中的所有文件
            string[] files = Directory.GetFiles(folder);
            foreach (string file in files)
            {
                File.Delete(file);
            }

            // 清空文件夹中的所有子文件夹
            string[] subfolders = Directory.GetDirectories(folder);
            foreach (string subfolder in subfolders)
            {
                Directory.Delete(subfolder, true);
            }
        }

        /// <summary>
        /// 复制文件夹
        /// </summary>
        /// <param name="sourceDirectory"></param>
        /// <param name="targetDirectory"></param>
        public static void CopyDirectory(string sourceDirectory, string targetDirectory)
        {
            // 创建目标文件夹
            Directory.CreateDirectory(targetDirectory);

            // 获取源文件夹中的所有文件
            string[] files = Directory.GetFiles(sourceDirectory);

            // 拷贝文件
            foreach (string file in files)
            {
                string fileName = Path.GetFileName(file);
                string targetPath = Path.Combine(targetDirectory, fileName);
                File.Copy(file, targetPath, true);
            }

            // 获取源文件夹中的所有子文件夹
            string[] directories = Directory.GetDirectories(sourceDirectory);

            // 递归拷贝子文件夹
            foreach (string directory in directories)
            {
                if (directory.Equals(targetDirectory))
                {
                    continue;
                }
                string directoryName = Path.GetFileName(directory);
                string targetSubDirectory = Path.Combine(targetDirectory, directoryName);
                CopyDirectory(directory, targetSubDirectory);
            }
        }
    }
}
